/*  The Librarian

    Copyright © 2005 Debarshi Ray

    This file is part of 'The Librarian'.

    'The Librarian' is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    'The Librarian' is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with 'The Librarian'; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 

    Written by: Debarshi Ray <debarshi.ray@gmail.com>
*/


#include "Librarian.h"

extern int days[12];
extern struct FindDialog find_entry;
extern gboolean active_table,logged_in,table_open,updated[2];
extern GtkWidget *application_window,*table_window,*viewtable;
extern MYSQL *connection;

void CloseTable(GtkWindow *window,gpointer data)
 {
   if(table_open)
    {
     gtk_container_remove(GTK_CONTAINER(table_window),viewtable);
     table_open = FALSE;
    }

  return;
 }

void Find(GtkWidget *dialog,gint response,gpointer user_data)
 {
  int (*compare)(const char*,const char*,size_t);
  gboolean found=FALSE,within;
  static gboolean valid;
  gchar *author,buffer[128],*ID,*name,*publisher;
  static GtkTreeIter iter;
  static GtkTreeModel *tree_model;
  GtkTreePath *path;
  GtkTreeSelection *selection;
  
   if(response == GTK_RESPONSE_OK && table_open)
    {
      if(user_data)
       {
        strcpy(find_entry.search_string,gtk_entry_get_text(GTK_ENTRY((GtkWidget*) user_data)));
        tree_model = gtk_tree_view_get_model(GTK_TREE_VIEW(viewtable));
        valid = gtk_tree_model_get_iter_first(tree_model,&iter);
       }
      switch(active_table)
       {
         case 1:
           while(!found && valid)
            {
             gtk_tree_model_get(tree_model,&iter,MEMBER_COLUMN_ID,&ID,MEMBER_COLUMN_NAME,&name,-1);
              if(find_entry.match_entire_word_only)
                within = FALSE;
              else
                within = TRUE;
              if(find_entry.match_case)
                compare = strncmp;
              else
                compare = strncasecmp;
              if((found = searchstr(ID,find_entry.search_string,within,compare) || searchstr(name,find_entry.search_string,within,compare)))
               {
                path = gtk_tree_model_get_path(tree_model,&iter);
                selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(viewtable));
                gtk_tree_view_set_cursor(GTK_TREE_VIEW(viewtable),path,NULL,FALSE);
                gtk_tree_selection_select_iter(selection,&iter);
                gtk_tree_view_expand_to_path(GTK_TREE_VIEW(viewtable),path);
               }
             g_free(ID);
             g_free(name);
             valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(tree_model),&iter);
            }
         break;
         case 2:
           while(!found && valid)
            {
             gtk_tree_model_get(tree_model,&iter,TITLE_COLUMN_ID,&ID,TITLE_COLUMN_NAME,&name,TITLE_COLUMN_AUTHOR,&author,TITLE_COLUMN_PUBLISHER,&publisher,-1);
              if(find_entry.match_entire_word_only)
                within = FALSE;
              else
                within = TRUE;
              if(find_entry.match_case)
                compare = strncmp;
              else
                compare = strncasecmp;
              if((found = searchstr(ID,find_entry.search_string,within,compare) || searchstr(name,find_entry.search_string,within,compare) || searchstr(author,find_entry.search_string,within,compare) || searchstr(publisher,find_entry.search_string,within,compare)))
               {
                path = gtk_tree_model_get_path(tree_model,&iter);
                selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(viewtable));
                gtk_tree_view_set_cursor(GTK_TREE_VIEW(viewtable),path,NULL,FALSE);
                gtk_tree_selection_select_iter(selection,&iter);
                gtk_tree_view_expand_to_path(GTK_TREE_VIEW(viewtable),path);
               }
             g_free(ID);
             g_free(name);
             g_free(author);
             g_free(publisher);
             valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(tree_model),&iter);
            }
         break;
         default:
         break;
       }
      if(!found)
       {
        sprintf(buffer,"The text \"%s\" was not found.",find_entry.search_string);
        ShowMessage(GTK_WINDOW(dialog),buffer,GTK_MESSAGE_INFO);
       }
    }

   else if(response == GTK_RESPONSE_APPLY && table_open)
     FindNext(dialog,NULL);

   else if(response != GTK_RESPONSE_CANCEL && !table_open)
    {
     strcpy(buffer,"There is no table currently open.");
     ShowMessage(GTK_WINDOW(dialog),buffer,GTK_MESSAGE_ERROR);
    }

   else if(dialog && response == GTK_RESPONSE_CANCEL)
     gtk_widget_destroy(dialog);

  return;
 }

void FindNext(GtkWidget *dialog,gpointer user_data)
 {
  Find(GTK_IS_WINDOW(dialog)?dialog:NULL,GTK_RESPONSE_OK,NULL);

  return;
 }

void IssueTitle(GtkWidget *dialog,gint response,gpointer user_data)
 {  
  gchar member_ID[6],title_ID[35];
  gchar *buffer;
  int available,i,issued,returned;
  IssueTitleDialog *entry = (IssueTitleDialog *) user_data;
  struct MemberRecord member_record;
  struct MemberTitleRecord member_title_record;
  MYSQL_RES *mysql_res;
  MYSQL_ROW mysql_row;
  time_t raw_time;
  struct tm *user_time;
  struct TitleRecord title_record;
  struct TitleRecordDetails title_record_details;

   if(response == GTK_RESPONSE_ACCEPT)
    {
     buffer = (gchar*) malloc(sizeof(gchar)*1024);
     strcpy(member_ID,gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->member_ID_entry)));
     sprintf(buffer,"SELECT Issued,Returned FROM Member WHERE ID=\"%s\"",member_ID);
     mysql_res = MySQLQuery(buffer,TRUE);
      if(mysql_res)
       {
        mysql_row = mysql_fetch_row(mysql_res);
        issued = atoi(mysql_row[0]);
        returned = atoi(mysql_row[1]);
       }
      else
       {
        sprintf(buffer,"Member: Retrieve failed.\nRetrieve error %d: %s.",mysql_errno(connection),mysql_error(connection));
        ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
       }
     mysql_free_result(mysql_res);
      for(i=0;i<Maximum_Titles_Issued;++i)
       {
        strcpy(title_ID,gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_ID_entry[i])));
         if(strlen(title_ID) == 0)
          {printf("blank\n");
           continue;
          }
         if(issued - returned + i == 5)
          {
           sprintf(buffer,"%d can not borrow any more titles!",member_ID);
           ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
           break;
          }
        sprintf(buffer,"SELECT Availability FROM TitleDetail WHERE ID=\"%s\"",title_ID);
         if(mysql_query(connection,buffer))
          {
           sprintf(buffer,"TitleDetail: Selection failed.\nSelection error %d: %s.",mysql_errno(connection),mysql_error(connection));
           ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
          }
         else
          {
           mysql_res = mysql_use_result(connection);
            if(mysql_res)
             {
              mysql_row = mysql_fetch_row(mysql_res);
               if(mysql_row[0][0] != 'Y')
                {
                 ShowMessage(GTK_WINDOW(application_window),"This book is not available!",GTK_MESSAGE_ERROR);
                 mysql_free_result(mysql_res);
                 continue;           
                }
              mysql_free_result(mysql_res);
             }
            else
             {
              sprintf(buffer,"TitleDetail: Retrieve failed.\nRetrieve error %d: %s.",mysql_errno(connection),mysql_error(connection));
              ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
             }
          }         
        sprintf(buffer,"UPDATE Member SET Issued=%d WHERE ID=\"%s\"",++issued,member_ID);
        mysql_query(connection,buffer);
        sprintf(buffer,"UPDATE TitleDetail SET Availability='N' WHERE ID=\"%s\"",title_ID);
        mysql_query(connection,buffer);
        sprintf(buffer,"UPDATE TitleDetail SET Member=\"%s\" WHERE ID=\"%s\"",member_ID,title_ID);
        mysql_query(connection,buffer);
        sprintf(buffer,"SELECT Available FROM Title WHERE ID=\"%s\"",title_ID+2);
         if(mysql_query(connection,buffer))
          {
           sprintf(buffer,"Title: Selection failed.\nSelection error %d: %s.",mysql_errno(connection),mysql_error(connection));
           ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
          }
         else
          {
           mysql_res = mysql_use_result(connection);
            if(mysql_res)
             {
              mysql_row = mysql_fetch_row(mysql_res);
              available = atoi(mysql_row[0]);
              mysql_free_result(mysql_res);
             }
            else
             {
              sprintf(buffer,"Title: Retrieve failed.\nRetrieve error %d: %s.",mysql_errno(connection),mysql_error(connection));
              ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
             }
          }
        sprintf(buffer,"UPDATE Title SET Available=%d WHERE ID=\"%s\"",available-1,title_ID+2);
        mysql_query(connection,buffer);
       }
    }
  
  gtk_widget_destroy(dialog);

  return;
 }

void Login(GtkWidget *dialog,gint response,gpointer user_data)
 {
  gtk_widget_destroy(dialog);

   if(response == GTK_RESPONSE_OK)
    {
      if(MySQLServerConnect(gnome_password_dialog_get_username(GNOME_PASSWORD_DIALOG(user_data)),gnome_password_dialog_get_password(GNOME_PASSWORD_DIALOG(user_data)),gnome_password_dialog_get_domain(GNOME_PASSWORD_DIALOG(user_data))))
        MakeLoginDialog();
    }

   if(!logged_in)
     logged_in = TRUE;

  return;
 }

void Logout(void)
 {
   if(logged_in)
    {
     logged_in = FALSE;
     CloseTable(NULL,NULL);
     MySQLServerDisconnect();
    }

  return;
 }

void MatchEntireWordOnly(GtkWidget *checkbutton,gpointer user_data)
 {
  find_entry.match_entire_word_only = !find_entry.match_entire_word_only;
  
  return;
 }

void NewMember(GtkWidget *dialog,gint response,gpointer user_data)
 {
  gchar sex[3] = "-MF";
  gchar *buffer;
  int number_of_members=0,number_of_titles;
  MYSQL_RES *mysql_res;
  MYSQL_ROW mysql_row;
  struct NewMemberDialog *entry = (struct NewMemberDialog *) user_data;

   if(response == GTK_RESPONSE_ACCEPT)
    {
     buffer = (gchar*) malloc(sizeof(gchar)*1024);
     sprintf(buffer,"INSERT INTO Member(ID,Name,Sex,Age,Address,Issued,Returned) VALUES(\"%s\",\"%s\",\'%c\',%s,\"%s\",0,0)",gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->member_ID_entry)),gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->member_name_entry)),sex[gtk_combo_box_get_active(GTK_COMBO_BOX((GtkWidget*) entry->member_sex_combo))],gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->member_age_spin)),gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->member_address_entry)));
     printf("%s\n",buffer);
      if(mysql_query(connection,buffer))
       {
        sprintf(buffer,"Member: Insertion failed.\nInsertion error %d: %s.",mysql_errno(connection),mysql_error(connection));
        ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
       }
      if(table_open && active_table == 1)
        updated[active_table] = TRUE;
     free(buffer);
    }
  
  gtk_widget_destroy(dialog);

  return;
 }

void NewTitle(GtkWidget *dialog,gint response,gpointer user_data)
 {
  gchar pre[3];
  gchar *buffer;
  FILE *input,*output;
  GtkWidget *message;
  int copies,i,number_of_members=0,number_of_titles;
  unsigned long title_ID;
  MYSQL_RES *mysql_res;
  MYSQL_ROW mysql_row;
  struct NewTitleDialog *entry = (struct NewTitleDialog *) user_data;
  struct TitleRecord new_title;
  struct TitleRecordDetails new_title_details;
  struct UniversalRecord information; 

   if(response == GTK_RESPONSE_ACCEPT)
    {
     buffer = (gchar*) malloc(sizeof(gchar)*1024);
     copies = atoi(gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_copies_spin)));
     sprintf(buffer,"INSERT INTO Title(Name,Author,Publisher,ISBN,Reader_Level,Shelving_Category,Copies,Available,Price) VALUES(\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",%d,%d,%s)",gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_name_entry)),gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_author_entry)),gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_publisher_entry)),gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_ISBN_entry)),gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_reader_level_entry)),gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_shelving_category_entry)),copies,copies,gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_price_spin)));
      if(mysql_query(connection,buffer))
       {
        sprintf(buffer,"Title: Insertion failed.\nInsertion error %d: %s.",mysql_errno(connection),mysql_error(connection));
        ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
       }
      else
       {
        title_ID = mysql_insert_id(connection);
         for(i=0;i<copies;++i)
          {
           sprintf(buffer,"INSERT INTO TitleDetail(TitleID,ID,Availability) VALUES(%d,\"%c%c%d\",'Y')",title_ID,title_ID,65+i/26,65+i%26);
            if(mysql_query(connection,buffer))
             {
              sprintf(buffer,"TitleDetail: Insertion failed.\nInsertion error %d: %s.",mysql_errno(connection),mysql_error(connection));
              ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
             }
          }
       }
      if(table_open && active_table == 2)
        updated[active_table] = TRUE;
     free(buffer);
    }
  
  gtk_widget_destroy(dialog);

  return;
 }

void QuickFind(GtkWidget *button,gpointer user_data)
 {
  find_entry.match_case = FALSE;
  find_entry.match_entire_word_only = FALSE;
  Find(NULL,GTK_RESPONSE_OK,user_data);

  return;
 }

void RefreshTable(GtkWidget *dialog,gpointer user_data)
 {
  GtkWidget *message;

   if(table_open && updated[active_table])
    {
     message = gtk_message_dialog_new(GTK_WINDOW(application_window),0,GTK_MESSAGE_INFO,GTK_BUTTONS_NONE,"Refreshing current table.\nPlease wait...");
     CloseTable(NULL,NULL);
      if(active_table == 1)
        MakeMemberTableTree(NULL,NULL);
      else if(active_table == 2)
        MakeTitleTableTree(NULL,NULL);
     gtk_widget_destroy(message);
    }

  return;
 }

void ReturnTitle(GtkWidget *dialog,gint response,gpointer user_data)
 {  
  char valid=0;
  char member_ID[8],title_ID[16];
  FILE *member_file,*member_title_file,*title_file;
  int count=0,i,j,k;
  long int position;
  ReturnTitleDialog *entry = (ReturnTitleDialog *) user_data;
  struct MemberRecord member_record;
  struct MemberTitleRecord member_title_record;
  time_t raw_time;
  struct TitleRecord title_record;
  struct TitleRecordDetails title_record_details;
  struct tm *user_time;

   if(response == GTK_RESPONSE_ACCEPT)
    {
     strcpy(member_ID,gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->member_ID_entry)));
      if((member_file = fopen(MemberFile,"rb+")) != NULL && (member_title_file = fopen(member_ID,"rb+")) != NULL && (title_file = fopen(TitleFile,"rb+")) != NULL)
       {
         for(i=0;1;++i)
          {
            if(fread((void*)&member_record,sizeof(struct MemberRecord),1,member_file) != 1)
              break;
            
            if(strcmp(member_record.ID,member_ID) == 0)
              {
               printf("Member ID validation passed!\n");
               valid = 1;
               break;
              }
          }
        fseek(member_file,i*sizeof(struct MemberRecord),SEEK_SET);
         for(i=0;i<Maximum_Titles_Issued;++i,rewind(title_file),rewind(member_title_file))
          {
           strcpy(title_ID,gtk_entry_get_text(GTK_ENTRY((GtkWidget*) entry->title_ID_entry[i])));
            if(title_ID[0] != '\0')
             {
               for(j=0,valid=FALSE;valid!=TRUE;++j)
                {
                  if(fread((void*)&title_record,sizeof(struct TitleRecord),1,title_file) != 1)
                    break;
                  if(strcmp(title_ID+2,title_record.ID) == 0)
                   {
                     for(k=0;k<title_record.copies;++k)
                      {
                       position = ftell(title_file);
                        if(fread((void*)&title_record_details,sizeof(struct TitleRecordDetails),1,title_file) == 1)
                         {
                           if(strcmp(title_record_details.ID,title_ID) == 0 && strcmp(title_record_details.member,member_ID) == 0)
                            {
                             printf("Title ID located.\n");
                             valid = TRUE;
                             break;
                            }
                         }
                      }
                   }
                  else
                    fseek(title_file,title_record.copies*sizeof(struct TitleRecordDetails),SEEK_CUR);
                }
               if(!valid)
                {
                 printf("Title ID could not be located.\n");
                 continue;
                }
              fseek(title_file,position,SEEK_SET);
              raw_time = time(NULL);
              user_time = localtime(&raw_time);
//              strcpy(title_record_details.member,"");
              strftime(title_record_details.return_date,sizeof(title_record_details.return_date),"%d %b %Y",user_time);
              fwrite((void*)&title_record_details,sizeof(struct TitleRecordDetails),1,title_file);
               for(j=0,valid=FALSE;valid!=TRUE;++j)
                {
                 position = ftell(member_title_file);
                  if(fread((void*)&member_title_record,sizeof(struct MemberTitleRecord),1,member_title_file) != 1)
                    break;
                  if(strcmp(title_ID,member_title_record.ID) == 0)
                   {
                    printf("Title ID validation passed!\n");
                    valid = TRUE;
                    break;
                   }
                }
               if(!valid)
                {
                 printf("Title ID validation failed!\n");
                 continue;
                }
              fseek(member_title_file,position,SEEK_SET);
              strftime(member_title_record.return_date,sizeof(member_title_record.return_date),"%d %b %Y",user_time);
              fwrite((void*)&member_title_record,sizeof(struct MemberTitleRecord),1,member_title_file);
              ++count;
             }
          }
        member_record.titles_pending-=count;
        fwrite((void*)&member_record,sizeof(struct MemberRecord),1,member_file);
       }
     printf("%d titles returned.\n",count);
      if(table_open && (active_table == 1) || (active_table == 2))
        updated[1] = updated[2] = TRUE;
     fclose(member_file);
     fclose(member_title_file);
     fclose(title_file);
    }
  
  gtk_widget_destroy(dialog);

  return;
 }
