/*  The Librarian

    Copyright © 2005 Debarshi Ray

    This file is part of 'The Librarian'.

    'The Librarian' is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    'The Librarian' is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with 'The Librarian'; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 

    Written by: Debarshi Ray <debarshi.ray@gmail.com>
*/


#include "Librarian.h"

extern struct FindDialog find_entry;
extern gboolean active_table,logged_in,table_open,updated[2];
extern GtkWidget *application_bar,*application_window,*table_window,*viewtable;
extern MYSQL *connection;

static GnomeUIInfo usermenu[] = {
  GNOMEUIINFO_ITEM_STOCK("Log In...","Log in to your account",MakeLoginDialog,GNOME_STOCK_AUTHENTICATION),
  GNOMEUIINFO_ITEM_STOCK("Log Out...","Log out of this account to log in as a different user",Logout,GNOME_STOCK_MENU_QUIT),
  GNOMEUIINFO_MENU_QUIT_ITEM(CloseApplication,NULL),
  GNOMEUIINFO_END
 };

static GnomeUIInfo tables[] = {
  GNOMEUIINFO_ITEM_NONE("Members","Table containing the members' records",MakeMemberTableTree),
  GNOMEUIINFO_ITEM_NONE("Titles","Table containing the titles' records",MakeTitleTableTree),
  GNOMEUIINFO_END
 };

static GnomeUIInfo tablemenu[] = {
  {GNOME_APP_UI_SUBTREE,"_Open","Open a table",tables,NULL,NULL,GNOME_APP_PIXMAP_STOCK,GTK_STOCK_OPEN,0,(GdkModifierType) 0,NULL},
  GNOMEUIINFO_SEPARATOR,
  {GNOME_APP_UI_ITEM,"_Close","Close the current table",(gpointer) CloseTable,(gpointer) NULL,NULL,GNOME_APP_PIXMAP_STOCK,GTK_STOCK_CLOSE,'W',(GdkModifierType) GDK_CONTROL_MASK,NULL},
  GNOMEUIINFO_END
 };

static GnomeUIInfo titlesmenu[] = {
  {GNOME_APP_UI_ITEM,"_New","Add a new title",(gpointer) MakeNewTitleDialog,(gpointer) NULL,NULL,GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_BOOK_OPEN,'T',(GdkModifierType) GDK_CONTROL_MASK,NULL},
  {GNOME_APP_UI_ITEM,"_Issue","Issue title(s) to a member",(gpointer) MakeIssueTitleDialog,(gpointer) NULL,NULL,GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_BOOK_GREEN,'I',(GdkModifierType) GDK_CONTROL_MASK,NULL},
  {GNOME_APP_UI_ITEM,"R_e-issue","Re-issue title(s) to a member",(gpointer) NULL,(gpointer) NULL,NULL,GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_BOOK_BLUE,'U',(GdkModifierType) GDK_CONTROL_MASK,NULL},
 {GNOME_APP_UI_ITEM,"Re_turn","Return title(s) from a member",(gpointer) MakeReturnTitleDialog,(gpointer) NULL,NULL,GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_BOOK_RED,'Y',(GdkModifierType) GDK_CONTROL_MASK,NULL},
 {GNOME_APP_UI_ITEM,"_Remove","Remove an existing title",(gpointer) NULL,(gpointer) NULL,NULL,GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_BOOK_YELLOW,'T',(GdkModifierType) GDK_CONTROL_MASK | GDK_SHIFT_MASK,NULL},
  GNOMEUIINFO_END
 };

static GnomeUIInfo membersmenu[] = {
  {GNOME_APP_UI_ITEM,"_New","Add a new member",(gpointer) MakeNewMemberDialog,(gpointer) NULL,NULL,GNOME_APP_PIXMAP_STOCK,GTK_STOCK_ADD,'M',(GdkModifierType) GDK_CONTROL_MASK,NULL},
  {GNOME_APP_UI_ITEM,"_Remove","Remove an existing member",(gpointer) NULL,(gpointer) NULL,NULL,GNOME_APP_PIXMAP_STOCK,GTK_STOCK_REMOVE,'M',(GdkModifierType) GDK_CONTROL_MASK | GDK_SHIFT_MASK,NULL},
  GNOMEUIINFO_END
 };

static GnomeUIInfo searchmenu[] = {
  GNOMEUIINFO_MENU_FIND_ITEM(MakeFindDialog, NULL),
  GNOMEUIINFO_MENU_FIND_AGAIN_ITEM(FindNext, NULL),
  GNOMEUIINFO_END
 };

static GnomeUIInfo viewmenu[] = {
 {GNOME_APP_UI_TOGGLEITEM,"Toolbar","Change the visibility of the toolbar in the current window",(gpointer) NULL,NULL,NULL,GNOME_APP_PIXMAP_NONE,NULL,0,(GdkModifierType) 0,NULL},
{GNOME_APP_UI_TOGGLEITEM,"Statusbar","Change the visibility of the statusbar in the current window",(gpointer) NULL,NULL,NULL,GNOME_APP_PIXMAP_NONE,NULL,0,(GdkModifierType) 0,NULL},
  GNOMEUIINFO_SEPARATOR,
 {GNOME_APP_UI_ITEM,"_Refresh","Display the latest contents of the current table",(gpointer) RefreshTable,(gpointer) NULL,NULL,GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_REFRESH,'R',(GdkModifierType) GDK_CONTROL_MASK,NULL},
  GNOMEUIINFO_END
 };

static GnomeUIInfo helpmenu[] = {
  GNOMEUIINFO_MENU_ABOUT_ITEM(About,NULL),
  GNOMEUIINFO_END
 };

static GnomeUIInfo menubar[] = {
  {GNOME_APP_UI_SUBTREE_STOCK,N_("_User"),NULL,usermenu,NULL,NULL,(GnomeUIPixmapType) 0,NULL,0,(GdkModifierType) 0,NULL},
  {GNOME_APP_UI_SUBTREE_STOCK,N_("_Table"),NULL,tablemenu,NULL,NULL,(GnomeUIPixmapType) 0,NULL,0,(GdkModifierType) 0,NULL},
  {GNOME_APP_UI_SUBTREE_STOCK,N_("_Member"),NULL,membersmenu,NULL,NULL,(GnomeUIPixmapType) 0,NULL,0,(GdkModifierType) 0,NULL},
  {GNOME_APP_UI_SUBTREE_STOCK,N_("_Title"),NULL,titlesmenu,NULL,NULL,(GnomeUIPixmapType) 0,NULL,0,(GdkModifierType) 0,NULL},
  GNOMEUIINFO_MENU_EDIT_TREE(searchmenu),
  GNOMEUIINFO_MENU_VIEW_TREE(viewmenu),
  GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
  GNOMEUIINFO_END
 };


void About(GtkMenuItem *menuitem,gpointer data)
 {
  const char *author[] = {"Debarshi Ray     <debarshi.ray@gmail.com>",NULL};
  GdkPixbuf *icon = gdk_pixbuf_new_from_file("devhelp.png",NULL);
  GtkWidget *about = gtk_about_dialog_new();

  gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(about),"The Librarian");
  gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about),"0.1 (alpha)");
  gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about),"Copyright © 2005 Debarshi Ray");
  gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about),"Library management package for the GNOME desktop.");
  gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about),"'The Librarian' is free software; you can redistribute it and/or modify\nit under the terms of the GNU General Public License as published by\nthe Free Software Foundation; either version 2 of the License, or\n(at your option) any later version.\n\n'The Librarian' is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License\nalong with 'The Librarian'; if not, write to the Free Software Foundation, Inc.,\n51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA");
  gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(about),author);
  gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(about),icon);

  gtk_widget_show(about);

  return;
 }

void CloseApplication(GtkWindow *window,gpointer data)
 {
  Logout();
  gtk_main_quit();

  return;
 }

void MakeFindDialog(GtkMenuItem *menuitem,gpointer data)
 {
  gchar buffer[128];
  GtkWidget *find_dialog,*match_case,*match_entire_word_only,*search_string_entry;

  find_dialog = gtk_dialog_new_with_buttons("Find...",GTK_WINDOW(application_window),GTK_DIALOG_NO_SEPARATOR,GTK_STOCK_CLOSE,GTK_RESPONSE_CANCEL,"Find Next",GTK_RESPONSE_APPLY,GTK_STOCK_FIND,GTK_RESPONSE_OK,NULL);
  gtk_window_set_resizable(GTK_WINDOW(find_dialog),FALSE);
  search_string_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(search_string_entry),127);
  gtk_entry_set_width_chars(GTK_ENTRY(search_string_entry),31);
  gtk_entry_set_text(GTK_ENTRY(search_string_entry),find_entry.search_string);
  match_case = gtk_check_button_new_with_label("Match case");
  gtk_toggle_button_set_active((GtkToggleButton*)match_case,find_entry.match_case);
  match_entire_word_only = gtk_check_button_new_with_label("Match entire word only");
  gtk_toggle_button_set_active((GtkToggleButton*)match_entire_word_only,find_entry.match_entire_word_only);
  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(find_dialog)->vbox),10);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(find_dialog)->vbox),"Search for",search_string_entry);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(find_dialog)->vbox),match_case,FALSE,FALSE,1);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(find_dialog)->vbox),match_entire_word_only,FALSE,FALSE,1);
  g_signal_connect(GTK_OBJECT(match_case),"toggled",GTK_SIGNAL_FUNC(MatchCase),NULL);
  g_signal_connect(GTK_OBJECT(match_entire_word_only),"toggled",GTK_SIGNAL_FUNC(MatchEntireWordOnly),NULL);
  g_signal_connect(GTK_OBJECT(find_dialog),"response",GTK_SIGNAL_FUNC(Find),search_string_entry);
  gtk_widget_show_all(find_dialog);
   if(!table_open)
    {
     strcpy(buffer,"There is no table currently open.");
     ShowMessage(GTK_WINDOW(find_dialog),buffer,GTK_MESSAGE_WARNING);
    }

  return;
 }

void MakeIssueTitleDialog(GtkMenuItem *menuitem,gpointer data)
 {
  char buffer[16];
  GtkWidget *issue_title_dialog;
  int i;
  static IssueTitleDialog entry;

  issue_title_dialog = gtk_dialog_new_with_buttons("Issue...",GTK_WINDOW(application_window),GTK_DIALOG_MODAL|GTK_DIALOG_NO_SEPARATOR,GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,NULL);
  gtk_window_set_resizable(GTK_WINDOW(issue_title_dialog),FALSE);
  entry.member_ID_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.member_ID_entry),7);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.member_ID_entry),7);
   for(i=0;i<Maximum_Titles_Issued;entry.title_ID_entry[i++]=gtk_entry_new());
  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(issue_title_dialog)->vbox),10);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(issue_title_dialog)->vbox),"Member ID",entry.member_ID_entry);
   for(i=0;i<Maximum_Titles_Issued;++i)
    {
     sprintf(buffer,"Title%d ID",i);
     gtk_entry_set_max_length(GTK_ENTRY(entry.title_ID_entry[i]),15);
     gtk_entry_set_width_chars(GTK_ENTRY(entry.title_ID_entry[i]),15);
     add_widget_with_label(GTK_BOX(GTK_DIALOG(issue_title_dialog)->vbox),buffer,entry.title_ID_entry[i]);
    }
  g_signal_connect(GTK_OBJECT(issue_title_dialog),"response",GTK_SIGNAL_FUNC(IssueTitle),&entry);
  gtk_widget_show_all(issue_title_dialog);
  printf("Titles shall be issued in a shortwhile!\n");

  return;
 }

void MakeLoginDialog(void)
 {
  GtkWidget *login_dialog;

   if(logged_in)
    {
     Logout();
     logged_in = FALSE;
    }

  login_dialog = gnome_password_dialog_new("Login...","Login to your account.",NULL,NULL,FALSE);
  gnome_password_dialog_set_show_domain(GNOME_PASSWORD_DIALOG(login_dialog),TRUE);

  g_signal_connect(GTK_OBJECT(login_dialog),"response",GTK_SIGNAL_FUNC(Login),login_dialog);

  gtk_widget_show_all(login_dialog);

  return;
}

void MakeMemberTableTree(GtkMenuItem *menuitem,gpointer data)
 {
  char buffer[26];
  FILE *member_file,*member_title_file,*universal_file;
  GtkCellRenderer *renderer;
  GtkTreeIter *parent_iter,**child_iter;
  GtkTreeModel *sort_model;
  GtkTreeStore *store;
  GtkTreeViewColumn *column;
  int i,j,number_of_members=0,position;
  MYSQL_RES *mysql_res;
  MYSQL_ROW mysql_row;
  struct MemberRecord member_record;
  struct MemberTitleRecord member_title_record;
  struct UniversalRecord universal_record;

   if(table_open)
     CloseTable(NULL,NULL);

  mysql_res = MySQLQuery("SELECT COUNT(*) FROM Member",TRUE);
   if(mysql_res)
    {
     mysql_row = mysql_fetch_row(mysql_res);
     number_of_members = atoi(mysql_row[0]);
     gnome_appbar_set_progress_percentage(GNOME_APPBAR(application_bar),(float) 1 /(number_of_members+2));
    }
   else
    {
     sprintf(buffer,"Retrieve failed.\nRetrieve error %d: %s.",mysql_errno(connection),mysql_error(connection));
     ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
    }
  mysql_free_result(mysql_res);

  store = gtk_tree_store_new(MEMBER_N_COLUMNS,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);

   if(number_of_members > 0)
    {
      if(mysql_query(connection,"SELECT ID,Name FROM Member"))
       {
        sprintf(buffer,"Member: Selection failed.\nSelection error %d: %s.",mysql_errno(connection),mysql_error(connection));
        ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
        return;
       }
      else
       {
        mysql_res = mysql_use_result(connection);
        parent_iter = (GtkTreeIter*) malloc(sizeof(GtkTreeIter)*number_of_members);
        child_iter = (GtkTreeIter**) malloc(sizeof(GtkTreeIter*)*number_of_members);
         for(i=0;i<number_of_members;++i)
          {
           gtk_tree_store_append(store,parent_iter+i,NULL); 
           mysql_row = mysql_fetch_row(mysql_res);
           gnome_appbar_set_progress_percentage(GNOME_APPBAR(application_bar),(float) (i+2)/(number_of_members+2));
           printf("Row fetched\n");
           child_iter[i] = (GtkTreeIter*) malloc(sizeof(GtkTreeIter)*0);
           gtk_tree_store_set(store,parent_iter+i,MEMBER_COLUMN_ID,mysql_row[0],MEMBER_COLUMN_NAME,mysql_row[1],-1);
          }
        mysql_free_result(mysql_res);
       }
    }
  sort_model = gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(store));
  viewtable = gtk_tree_view_new_with_model(sort_model);
  renderer = gtk_cell_renderer_text_new();

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),MEMBER_COLUMN_ID,"Member/Title ID",renderer,"text",MEMBER_COLUMN_ID,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,MEMBER_COLUMN_ID);

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),MEMBER_COLUMN_NAME,"Name of Member/Title",renderer,"text",MEMBER_COLUMN_NAME,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_expand(column,TRUE);
  gtk_tree_view_column_set_sort_column_id(column,MEMBER_COLUMN_NAME);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),MEMBER_COLUMN_STATUS,"Status",renderer,"text",MEMBER_COLUMN_STATUS,NULL);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),MEMBER_COLUMN_DUE_DATE,"Due Date",renderer,"text",MEMBER_COLUMN_DUE_DATE,NULL);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),MEMBER_COLUMN_ISSUE_DATE,"Issue Date",renderer,"text",MEMBER_COLUMN_ISSUE_DATE,NULL);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),MEMBER_COLUMN_RETURN_DATE,"Return Date",renderer,"text",MEMBER_COLUMN_RETURN_DATE,NULL);

  gnome_appbar_set_progress_percentage(GNOME_APPBAR(application_bar),1.0);

  active_table = 1;
  gtk_container_add(GTK_CONTAINER(table_window),viewtable);
  gtk_widget_show_all(application_window);
  table_open = TRUE;
  updated[active_table] = FALSE;

  gnome_appbar_set_progress_percentage(GNOME_APPBAR(application_bar),0.0);

  return;
 }

void MakeNewMemberDialog(GtkMenuItem *menuitem,gpointer data)
 {
  static struct NewMemberDialog entry;
  GtkWidget *new_member_dialog;

  new_member_dialog = gtk_dialog_new_with_buttons("New Member...",GTK_WINDOW(application_window),GTK_DIALOG_MODAL|GTK_DIALOG_NO_SEPARATOR,GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,NULL);
  gtk_window_set_resizable(GTK_WINDOW(new_member_dialog),FALSE);
  entry.member_ID_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.member_ID_entry),7);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.member_ID_entry),7);
  entry.member_name_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.member_ID_entry),63);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.member_ID_entry),15);
  entry.member_address_entry = gtk_entry_new();
  entry.member_sex_combo = gtk_combo_box_new_text();
  gtk_combo_box_append_text(GTK_COMBO_BOX(entry.member_sex_combo),"----");
  gtk_combo_box_append_text(GTK_COMBO_BOX(entry.member_sex_combo),"Male");
  gtk_combo_box_append_text(GTK_COMBO_BOX(entry.member_sex_combo),"Female");
  gtk_combo_box_set_active(GTK_COMBO_BOX(entry.member_sex_combo),0);
  entry.member_age_spin = gtk_spin_button_new_with_range(1,26*27,1);
  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(new_member_dialog)->vbox),10);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_member_dialog)->vbox),"Member ID",entry.member_ID_entry);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_member_dialog)->vbox),"Name",entry.member_name_entry);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_member_dialog)->vbox),"Sex",entry.member_sex_combo);
 add_widget_with_label(GTK_BOX(GTK_DIALOG(new_member_dialog)->vbox),"Age",entry.member_age_spin);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_member_dialog)->vbox),"Permanent Address",entry.member_address_entry);
  g_signal_connect(GTK_OBJECT(new_member_dialog),"response",GTK_SIGNAL_FUNC(NewMember),&entry);
  gtk_widget_show_all(new_member_dialog);
  printf("Member will be added in a shortwhile!\n");

  return;
 }

void MakeNewTitleDialog(GtkMenuItem *menuitem,gpointer data)
 {
  static struct NewTitleDialog entry;
  GtkWidget *hbox,*new_title_dialog;

  new_title_dialog = gtk_dialog_new_with_buttons("New Title...",GTK_WINDOW(application_window),GTK_DIALOG_MODAL|GTK_DIALOG_NO_SEPARATOR,GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,NULL);
  gtk_window_set_resizable(GTK_WINDOW(new_title_dialog),FALSE);
  entry.title_name_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.title_name_entry),127);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.title_name_entry),31);
  entry.title_author_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.title_author_entry),63);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.title_author_entry),15);
  entry.title_publisher_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.title_publisher_entry),127);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.title_publisher_entry),31);
  entry.title_ISBN_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.title_ISBN_entry),15);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.title_ISBN_entry),15);
  entry.title_reader_level_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.title_reader_level_entry),127);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.title_reader_level_entry),31);
  entry.title_shelving_category_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.title_shelving_category_entry),127);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.title_shelving_category_entry),31);
  entry.title_copies_spin = gtk_spin_button_new_with_range(1,26*27,1);
  entry.title_price_spin = gtk_spin_button_new_with_range(0,1<<sizeof(gdouble)-1,1);
  hbox = gtk_hbox_new(TRUE,1);
  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(new_title_dialog)->vbox),10);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(new_title_dialog)->vbox),hbox,TRUE,TRUE,1);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_title_dialog)->vbox),"Name",entry.title_name_entry);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_title_dialog)->vbox),"Author(s)",entry.title_author_entry);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_title_dialog)->vbox),"Publisher",entry.title_publisher_entry);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_title_dialog)->vbox),"ISBN",entry.title_ISBN_entry);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_title_dialog)->vbox),"Reader Level",entry.title_reader_level_entry);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(new_title_dialog)->vbox),"Shelving Category",entry.title_shelving_category_entry);
  add_widget_with_label(GTK_BOX(hbox),"Copies",entry.title_copies_spin);
  add_widget_with_label(GTK_BOX(hbox),"Price",entry.title_price_spin);
  g_signal_connect(GTK_OBJECT(new_title_dialog),"response",GTK_SIGNAL_FUNC(NewTitle),(gpointer)&entry);
  gtk_widget_show_all(new_title_dialog);
  printf("Title will be added in a shortwhile!\n");

  return;
 }

void MakeReturnTitleDialog(GtkMenuItem *menuitem,gpointer data)
 {
  char buffer[16];
  GtkWidget *return_title_dialog;
  int i;
  static ReturnTitleDialog entry;

  return_title_dialog = gtk_dialog_new_with_buttons("Return...",GTK_WINDOW(application_window),GTK_DIALOG_MODAL|GTK_DIALOG_NO_SEPARATOR,GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,NULL);
  gtk_window_set_resizable(GTK_WINDOW(return_title_dialog),FALSE);
  entry.member_ID_entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry.member_ID_entry),7);
  gtk_entry_set_width_chars(GTK_ENTRY(entry.member_ID_entry),7);
   for(i=0;i<Maximum_Titles_Issued;entry.title_ID_entry[i++]=gtk_entry_new());
  gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(return_title_dialog)->vbox),10);
  add_widget_with_label(GTK_BOX(GTK_DIALOG(return_title_dialog)->vbox),"Member ID",entry.member_ID_entry);
   for(i=0;i<Maximum_Titles_Issued;++i)
    {
     sprintf(buffer,"Title%d ID",i);
     gtk_entry_set_max_length(GTK_ENTRY(entry.title_ID_entry[i]),15);
     gtk_entry_set_width_chars(GTK_ENTRY(entry.title_ID_entry[i]),15);
     add_widget_with_label(GTK_BOX(GTK_DIALOG(return_title_dialog)->vbox),buffer,entry.title_ID_entry[i]);
    }
  g_signal_connect(GTK_OBJECT(return_title_dialog),"response",GTK_SIGNAL_FUNC(ReturnTitle),&entry);
  gtk_widget_show_all(return_title_dialog);
  printf("Titles shall be returned in a shortwhile!\n");

  return;
 }

void MakeTableOpenDialog(GtkMenuItem *menuitem,gpointer data)
 {
  char *filename;
  GtkFileFilter *file_filter;
  GtkResponseType response;
  GtkWidget *message,*open_table_dialog;

  file_filter = gtk_file_filter_new();
  gtk_file_filter_set_name(file_filter,"The Librarian Table (*.lbr)");
  gtk_file_filter_add_pattern(file_filter,"*.lbr");
  open_table_dialog = gtk_file_chooser_dialog_new("Open table...",GTK_WINDOW(application_window),GTK_FILE_CHOOSER_ACTION_OPEN,GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,GTK_STOCK_OPEN,GTK_RESPONSE_ACCEPT,NULL);
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(open_table_dialog),file_filter);
  response = gtk_dialog_run(GTK_DIALOG(open_table_dialog));
   if(response == GTK_RESPONSE_ACCEPT)
    {
     filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(open_table_dialog));
      if(table_open)
        CloseTable(NULL,NULL);
      if(strcmp(filename,MemberFile) == 0)
        MakeMemberTableTree(NULL,NULL);
      else if(strcmp(filename,TitleFile) == 0)
        MakeTitleTableTree(NULL,NULL);
      else
       {
        message = gtk_message_dialog_new(GTK_WINDOW(open_table_dialog),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,"You are not permitted to view or edit the contents of\n%s.",filename);
        gtk_dialog_run(GTK_DIALOG(message));
        gtk_widget_destroy(message);
       }
     g_free(filename);
    }

  gtk_widget_destroy(open_table_dialog);

  return;
 }

void MakeTitleTableTree(GtkMenuItem *menuitem,gpointer data)
 {
  gchar buffer[64];
  gchar *titleID;
  gboolean valid;
  GtkCellRenderer *renderer;
  GtkTreeIter *parent_iter,**child_iter,iter;
  GtkTreeModel *sort_model;
  GtkTreeStore *store;
  GtkTreeViewColumn *column;
  int i,j,number_of_titles=0,position;
  int *copies=NULL;
  MYSQL_RES *mysql_res;
  MYSQL_ROW mysql_row;
  struct TitleRecord title_record;
  struct TitleRecordDetails title_record_details;
  struct UniversalRecord universal_record;

   if(table_open)
     CloseTable(NULL,NULL);

  mysql_res = MySQLQuery("SELECT COUNT(*) FROM Title",TRUE);
   if(mysql_res)
    {
     mysql_row = mysql_fetch_row(mysql_res);
     number_of_titles = atoi(mysql_row[0]);
     gnome_appbar_set_progress_percentage(GNOME_APPBAR(application_bar),(float) 1 / (number_of_titles+2));
    }
   else
    {
     sprintf(buffer,"Retrieve failed.\nRetrieve error %d: %s.",mysql_errno(connection),mysql_error(connection));
     ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
    }
  mysql_free_result(mysql_res);

  store = gtk_tree_store_new(TITLE_N_COLUMNS,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);

   if(number_of_titles > 0)
    {
      if(mysql_query(connection,"SELECT * FROM Title"))
       {
        sprintf(buffer,"Title: Selection failed.\nSelection error %d: %s.",mysql_errno(connection),mysql_error(connection));
        ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
        return;
       }
      else
       {
        mysql_res = mysql_use_result(connection);
        parent_iter = (GtkTreeIter*) malloc(sizeof(GtkTreeIter)*number_of_titles);
        child_iter = (GtkTreeIter**) malloc(sizeof(GtkTreeIter*)*number_of_titles);
        copies = (int*) malloc(sizeof(int)*number_of_titles);
         for(i=0;mysql_res&&i<number_of_titles;++i)
          {
           gtk_tree_store_append(store,parent_iter+i,NULL);
           mysql_row = mysql_fetch_row(mysql_res);
           gnome_appbar_set_progress_percentage(GNOME_APPBAR(application_bar),(float) (i+2) / (number_of_titles+2));
           copies[i] = atoi(mysql_row[7]);
           child_iter[i] = (GtkTreeIter*) malloc(sizeof(GtkTreeIter)*copies[i]);
           sprintf(buffer,"%s/%s",mysql_row[8],mysql_row[7]);
           gtk_tree_store_set(store,parent_iter+i,TITLE_COLUMN_ID,mysql_row[0],TITLE_COLUMN_NAME,mysql_row[1],TITLE_COLUMN_AUTHOR,mysql_row[2],TITLE_COLUMN_PUBLISHER,mysql_row[3],TITLE_COLUMN_ISBN,mysql_row[4],TITLE_COLUMN_READER_LEVEL,mysql_row[5],TITLE_COLUMN_SHELVING_CATEGORY,mysql_row[6],TITLE_COLUMN_STATUS,buffer,TITLE_COLUMN_PRICE,mysql_row[9],-1);
          }
        mysql_free_result(mysql_res);
       }

      for(i=0;i<number_of_titles;++i)
       {
        gtk_tree_model_get(GTK_TREE_MODEL(store),parent_iter+i,TITLE_COLUMN_ID,&titleID,-1);
        sprintf(buffer,"SELECT * FROM TitleDetail WHERE TitleID = %s",titleID);
         if(mysql_query(connection,buffer))
          {
           sprintf(buffer,"TitleDetail: Selection failed.\nSelection error %d: %s.",mysql_errno(connection),mysql_error(connection));
           ShowMessage(GTK_WINDOW(application_window),buffer,GTK_MESSAGE_ERROR);
           return;
          }
         else
          {
           mysql_res = mysql_use_result(connection);
            for(j=0;mysql_res&&j<copies[i];++j)
             {
              mysql_row = mysql_fetch_row(mysql_res);
              gtk_tree_store_append(store,*(child_iter+i)+j,parent_iter+i);
               if(mysql_row[2][0] == 'Y')
                 strcpy(buffer,"Available");
               else
                 strcpy(buffer,"Unavailable");
              gtk_tree_store_set(store,*(child_iter+i)+j,TITLE_COLUMN_ID,mysql_row[1],TITLE_COLUMN_NAME,"-do-",TITLE_COLUMN_AUTHOR,"-do-",TITLE_COLUMN_PUBLISHER,"-do-",TITLE_COLUMN_STATUS,buffer,TITLE_COLUMN_MEMBER,mysql_row[3],TITLE_COLUMN_DUE_DATE,mysql_row[4],TITLE_COLUMN_ISSUE_DATE,mysql_row[5],TITLE_COLUMN_RETURN_DATE,mysql_row[6],-1);
             }
          }
        mysql_free_result(mysql_res);
       }
      if(copies)
        free(copies);
    }

  sort_model = gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(store));
  viewtable = gtk_tree_view_new_with_model(sort_model);
  renderer = gtk_cell_renderer_text_new();

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_ID,"Title ID",renderer,"text",TITLE_COLUMN_ID,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,TITLE_COLUMN_ID);

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_NAME,"Name of Title",renderer,"text",TITLE_COLUMN_NAME,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,TITLE_COLUMN_NAME);
  gtk_tree_view_column_set_expand(column,TRUE);

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_AUTHOR,"Author(s)",renderer,"text",TITLE_COLUMN_AUTHOR,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,TITLE_COLUMN_AUTHOR);
  gtk_tree_view_column_set_expand(column,TRUE);

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_PUBLISHER,"Published by",renderer,"text",TITLE_COLUMN_PUBLISHER,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,TITLE_COLUMN_PUBLISHER);

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_ISBN,"ISBN",renderer,"text",TITLE_COLUMN_ISBN,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,TITLE_COLUMN_ISBN);

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_READER_LEVEL,"Reader Level",renderer,"text",TITLE_COLUMN_READER_LEVEL,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,TITLE_COLUMN_READER_LEVEL);

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_SHELVING_CATEGORY,"Shelving Category",renderer,"text",TITLE_COLUMN_SHELVING_CATEGORY,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,TITLE_COLUMN_SHELVING_CATEGORY);

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_STATUS,"Status",renderer,"text",TITLE_COLUMN_STATUS,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,TITLE_COLUMN_STATUS);
  gtk_tree_view_column_set_expand(column,TRUE);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_MEMBER,"Borrowed by",renderer,"text",TITLE_COLUMN_MEMBER,NULL);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_DUE_DATE,"Due Date",renderer,"text",TITLE_COLUMN_DUE_DATE,NULL);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_ISSUE_DATE,"Issue Date",renderer,"text",TITLE_COLUMN_ISSUE_DATE,NULL);

  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_RETURN_DATE,"Return Date",renderer,"text",TITLE_COLUMN_RETURN_DATE,NULL);

  position = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(viewtable),TITLE_COLUMN_PRICE,"Price (Rs.)",renderer,"text",TITLE_COLUMN_PRICE,NULL);
  column = gtk_tree_view_get_column(GTK_TREE_VIEW(viewtable),position-1);
  gtk_tree_view_column_set_sort_column_id(column,TITLE_COLUMN_PRICE);

  gnome_appbar_set_progress_percentage(GNOME_APPBAR(application_bar),1.0);

  active_table = 2;
  gtk_container_add(GTK_CONTAINER(table_window),viewtable);
  gtk_widget_show_all(application_window);
  table_open = TRUE;
  updated[active_table] = FALSE;

  gnome_appbar_set_progress_percentage(GNOME_APPBAR(application_bar),0.0);

  return;
 }

void MatchCase(GtkWidget *checkbutton,gpointer user_data)
 {
  find_entry.match_case = !find_entry.match_case;
  
  return;
 }

void ShowMessage(GtkWindow *window,const char *message_text,GtkMessageType type)
 {
  GtkWidget *message_dialog = gtk_message_dialog_new(window,GTK_DIALOG_MODAL,type,GTK_BUTTONS_OK,message_text);
  gtk_dialog_run(GTK_DIALOG(message_dialog));
  gtk_widget_destroy(message_dialog);

  return;
 }

GtkWidget* MakeApplicationWindow(void)
 {
  GtkWidget *app,*hbox,*search_button,*search_label,*toolbar,*vbox;
  static GtkWidget *search_entry;

  app = gnome_app_new("Librarian","The Librarian");
  gtk_window_set_default_size(GTK_WINDOW(app),600,500);
  gtk_window_maximize(GTK_WINDOW(app));
  gtk_signal_connect(GTK_OBJECT (app),"destroy",GTK_SIGNAL_FUNC(CloseApplication),NULL);
                       
  gnome_app_create_menus(GNOME_APP(app),menubar);
  
  toolbar = gtk_toolbar_new();
  gnome_app_add_toolbar(GNOME_APP(app),GTK_TOOLBAR(toolbar),"toolbar",BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,BONOBO_DOCK_TOP,1,0,0);
  
  gtk_container_set_border_width(GTK_CONTAINER(toolbar),1);
  gtk_toolbar_set_style(GTK_TOOLBAR(toolbar),GTK_TOOLBAR_ICONS);

  gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_OPEN,"Open a table",NULL,GTK_SIGNAL_FUNC(MakeTableOpenDialog),NULL,-1);
  gtk_toolbar_insert_space(GTK_TOOLBAR(toolbar),1);
  gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_REFRESH,"Display the latest contents of the current table",NULL,GTK_SIGNAL_FUNC(RefreshTable),NULL,-1);
  gtk_toolbar_insert_space(GTK_TOOLBAR(toolbar),3);
  gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_CLOSE,"Close the current table",NULL,GTK_SIGNAL_FUNC(CloseTable),NULL,-1);
  gtk_toolbar_insert_stock(GTK_TOOLBAR(toolbar),GTK_STOCK_QUIT,"Quit the application",NULL,GTK_SIGNAL_FUNC(CloseApplication),NULL,-1);

  table_window = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(table_window),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(table_window),GTK_SHADOW_ETCHED_IN);

  search_label = gtk_label_new("Search for");
  search_entry = gtk_entry_new();
  search_button = gtk_button_new_from_stock(GTK_STOCK_FIND);
  g_signal_connect(GTK_OBJECT(search_button),"clicked",GTK_SIGNAL_FUNC(QuickFind),(gpointer)search_entry);

  application_bar = gnome_appbar_new(TRUE,TRUE,GNOME_PREFERENCES_NEVER);
  gnome_app_set_statusbar(GNOME_APP(app),application_bar);
  gnome_app_install_menu_hints(GNOME_APP(app),menubar);

  vbox = gtk_vbox_new(FALSE,1);
  hbox = gtk_hbox_new(FALSE,1);
  gtk_box_pack_start(GTK_BOX(hbox),search_label,FALSE,FALSE,1);
  gtk_box_pack_start(GTK_BOX(hbox),search_entry,TRUE,TRUE,1);
  gtk_box_pack_start(GTK_BOX(hbox),search_button,FALSE,FALSE,1);
  gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,1);
  gtk_box_pack_start(GTK_BOX(vbox),table_window,TRUE,TRUE,1);
  gnome_app_set_contents(GNOME_APP(app),vbox);

  return app;
 }
