/*  The Librarian

    Copyright © 2005 Debarshi Ray

    This file is part of 'The Librarian'.

    'The Librarian' is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    'The Librarian' is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with 'The Librarian'; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 

    Written by: Debarshi Ray <debarshi.ray@gmail.com>
*/


#ifndef __Librarian_H
#define __Librarian_H

#include <gtk/gtk.h>
#include <gnome.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include "/usr/include/mysql/mysql.h"

/************************************************************************************************/

#define Maximum_Titles_Issued 5
#define TitleFile "/home/rishi/My Projects/Title.lbr"
#define UniversalFile "/home/rishi/My Projects/Universal.lbr"
#define MemberFile "/home/rishi/My Projects/Member.lbr"

/************************************************************************************************/

enum {MEMBER_COLUMN_ID,MEMBER_COLUMN_NAME,MEMBER_COLUMN_STATUS,MEMBER_COLUMN_DUE_DATE,MEMBER_COLUMN_ISSUE_DATE,MEMBER_COLUMN_RETURN_DATE,MEMBER_N_COLUMNS};
enum {TITLE_COLUMN_ID,TITLE_COLUMN_NAME,TITLE_COLUMN_AUTHOR,TITLE_COLUMN_PUBLISHER,TITLE_COLUMN_ISBN,TITLE_COLUMN_READER_LEVEL,TITLE_COLUMN_SHELVING_CATEGORY,TITLE_COLUMN_STATUS,TITLE_COLUMN_MEMBER,TITLE_COLUMN_DUE_DATE,TITLE_COLUMN_ISSUE_DATE,TITLE_COLUMN_RETURN_DATE,TITLE_COLUMN_PRICE,TITLE_COLUMN_BOUGHT,TITLE_N_COLUMNS};

/************************************************************************************************/

struct FindDialog
 {
  gchar search_string[128];
  gboolean match_case;
  gboolean match_entire_word_only;
 };

struct IssueReturnTitleDialog
 {
  GtkWidget *member_ID_entry;
  GtkWidget *title_ID_entry[Maximum_Titles_Issued];
 };

struct LoginDialog
 {
  GtkWidget *hostname_entry;
  GtkWidget *password_entry;
  GtkWidget *username_entry;
 };

struct NewMemberDialog
 {
  GtkWidget *member_ID_entry;
  GtkWidget *member_name_entry;
  GtkWidget *member_sex_combo;
  GtkWidget *member_age_spin;
  GtkWidget *member_address_entry;
 };

struct NewTitleDialog
 {
  GtkWidget *title_name_entry;
  GtkWidget *title_author_entry;
  GtkWidget *title_publisher_entry;
  GtkWidget *title_ISBN_entry;
  GtkWidget *title_reader_level_entry;
  GtkWidget *title_shelving_category_entry;
  GtkWidget *title_copies_spin;
  GtkWidget *title_price_spin;
 };

struct MemberRecord
 {
  char ID[8];
  char name[64];
  char sex;
  char age;
  char address[256];
  int titles_issued;
  int titles_pending;
 };

struct MemberTitleRecord
 {
  char ID[16];
  char name[128];
  char due_date[12];
  char issue_date[12];
  char return_date[12];
 };

struct TitleRecord
 {
  char ID[16];
  char name[128];
  char author[64];
  char publisher[128];
  int copies;
 };

struct TitleRecordDetails
 {
  char ID[16];
  char member[16];
  char due_date[12];
  char issue_date[12];
  char return_date[12];
 };

struct UniversalRecord
 {
  int number_of_members;
  int number_of_titles;
 };

typedef struct IssueReturnTitleDialog IssueTitleDialog;
typedef struct IssueReturnTitleDialog ReturnTitleDialog;

/************************************************************************************************/

void CloseTable(GtkWindow *window,gpointer data);
void Find(GtkWidget *dialog,gint response,gpointer user_data);
void FindNext(GtkWidget *dialog,gpointer user_data);
void IssueTitle(GtkWidget *dialog,gint response,gpointer user_data);
void Login(GtkWidget *dialog,gint response,gpointer user_data);
void Logout(void);
void MatchCase(GtkWidget *checkbutton,gpointer user_data);
void MatchEntireWordOnly(GtkWidget *checkbutton,gpointer user_data);
void NewMember(GtkWidget *dialog,gint response,gpointer user_data);
void NewTitle(GtkWidget *dialog,gint response,gpointer user_data);
void QuickFind(GtkWidget *button,gpointer user_data);
void RefreshTable(GtkWidget *dialog,gpointer user_data);
void ReturnTitle(GtkWidget *dialog,gint response,gpointer user_data);
/************************************************************************************************/
void About(GtkMenuItem *menuitem,gpointer data);
void CloseApplication(GtkWindow *window,gpointer data);
void MakeFindDialog(GtkMenuItem *menuitem,gpointer data);
void MakeIssueTitleDialog(GtkMenuItem *menuitem,gpointer data);
void MakeLoginDialog(void);
void MakeMemberTableTree(GtkMenuItem *menuitem,gpointer data);
void MakeNewMemberDialog(GtkMenuItem *menuitem,gpointer data);
void MakeNewTitleDialog(GtkMenuItem *menuitem,gpointer data);
void MakeReturnTitleDialog(GtkMenuItem *menuitem,gpointer data);
void MakeTableOpenDialog(GtkMenuItem *menuitem,gpointer data);
void MakeTitleTableTree(GtkMenuItem *menuitem,gpointer data);
void ShowMessage(GtkWindow *window,const char *message_text,GtkMessageType type);
/************************************************************************************************/
GtkWidget* MakeApplicationWindow(void);
/************************************************************************************************/
void add_widget_with_label(GtkBox *box,gchar *caption,GtkWidget *widget);
void Initialize(void);
void** Malloc2D(long rows,long columns,size_t size);
gboolean searchstr(gchar *s1,gchar *s2,gboolean within,int (*compare)(const char*,const char*,size_t));

/************************************************************************************************/

MYSQL_RES* MySQLQuery(char *query,gboolean retrieve);
int MySQLServerConnect(char *username,char *password,char *hostname);
void MySQLServerDisconnect(void);

#endif
